/* findfile.c -- Find and display files.  I cheated -- there is no find code
 * in this program at all.  Instead, I have popen() use the GNU (or POSIX)
 * find to do the dirty work for me.  You /do/ have GNU findutils on your
 * system (and in your PATH), right?  ;)
 * Search for sprintf(shellcmd to find the command line definition. */

#include <Xm/Xm.h>
#include <Xm/BulletinB.h>
#include <Xm/CascadeB.h>
#include <Xm/Form.h>
#include <Xm/Label.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/Transfer.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "defines.h"
#include "errormsg.h"
#include "filecmd.h"
#include "getdir.h"
#include "getinfo.h"
#include "helpsys.h"
#include "menubars.h"
#include "permission.h"

extern Widget top_level;
extern char *home_dir;
extern char current_wd[120];

/* Text field for file search pattern. */
Widget find_tf1, find_tf2, f_slist;

/* Find menu callback.  Special handling for things in Find window. */
void FindMenuCallback(Widget w, XtPointer client_data, XtPointer call_data)
{
  XmStringTable selections;
  char *thisitem;
  int integers;

  XtVaGetValues(f_slist, XmNselectedItems, &selections, 
		XmNselectedItemCount, &integers, NULL);
  if (integers != 0) { /* There is a selected item */
  XmStringGetLtoR((XmString)selections[0], XmSTRING_DEFAULT_CHARSET, 
		  &thisitem);
  thisitem[strlen(thisitem)-1] = '\0';
#ifdef DEBUG
  printf("%s\n", thisitem);
#endif
  switch ((int) client_data)
    {
    case FILE_MENU_RUN:
      RunSelectedFile(thisitem);
      break;
    case FILE_MENU_COPY:
      CopyFile(thisitem);
      break;
    case FILE_MENU_MOVE:
      MoveFile(thisitem);
      break;
    case FILE_MENU_DELETE:
      DeleteFile(thisitem);
      break;
    case FILE_MENU_INFO:
      display_info(thisitem);
      break;      
    default:
      printf("Menu option not available yet\n");
      break;
    }
  }
}

/* General Cancel callback.  Destroy widget in client_data. */
void f_cancelCB(Widget w, XtPointer client_data, XtPointer call_data)
{
  XtDestroyWidget((Widget)client_data);
}

/* Load "/" into search directory field. */
void find_b1_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
  XtVaSetValues(find_tf2, XmNvalue, "/", NULL);
}

/* Load home directory into search directory field. */
void find_b2_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
  XtVaSetValues(find_tf2, XmNvalue, home_dir, NULL);
}

/* Prototype */
void find_CB(Widget, XtPointer, XtPointer);

/* Ask about what directories and file pattern we should look for. */
void FindFileDialog()
{
  Widget find_shell, find_bb, find_b1, find_b2, find_ok, find_cancel, 
         find_help, find_l1, find_l2;

  find_shell = XtVaCreatePopupShell("find_qshell", vendorShellWidgetClass,
				    top_level, XmNtitle, "Find file(s)", NULL);
  find_bb = XtVaCreateManagedWidget("find_bb", xmBulletinBoardWidgetClass,
				    find_shell, XmNwidth, 400, XmNheight, 200,
				    NULL);

  find_l1 = XtVaCreateManagedWidget("find_l1", xmLabelWidgetClass, find_bb,
				    XmNlabelString, XmStringCreateSimple("Find this file or pattern:"), XmNx, 5, XmNy, 5, NULL);
  find_tf1 = XtVaCreateManagedWidget("find_tf", xmTextFieldWidgetClass,
				     find_bb, XmNx, 5, XmNy, 35, 
				     XmNvalue, "*", XmNwidth, 270, NULL);
  XtAddCallback(find_tf1, XmNactivateCallback, find_CB,
		(XtPointer) find_shell);
  find_l2 = XtVaCreateManagedWidget("find_l2", xmLabelWidgetClass, find_bb,
				    XmNlabelString, XmStringCreateSimple("Start from this directory:"), XmNx, 5, XmNy, 73, NULL);
  find_tf2 = XtVaCreateManagedWidget("find_tf", xmTextFieldWidgetClass,
				     find_bb, XmNx, 5, XmNy, 100, 
				     XmNwidth, 270, 
				     XmNvalue, current_wd, NULL);
  XtAddCallback(find_tf2, XmNactivateCallback, find_CB,
		(XtPointer) find_shell);
  find_b1 = XtVaCreateManagedWidget("find_b1", xmPushButtonWidgetClass, 
				    find_bb, XmNx, 280, XmNy, 104,
				    XmNlabelString, XmStringCreateSimple("/"),
				    XmNwidth, 50, NULL);
  XtAddCallback(find_b1, XmNactivateCallback, find_b1_CB, NULL);
  find_b2 = XtVaCreateManagedWidget("find_b2", xmPushButtonWidgetClass,
				    find_bb, XmNx, 335, XmNy, 104,
				    XmNlabelString, XmStringCreateSimple("Home"), XmNwidth, 50, NULL);
  XtAddCallback(find_b2, XmNactivateCallback, find_b2_CB, NULL);

  find_ok = XtVaCreateManagedWidget("Find", xmPushButtonWidgetClass, find_bb,
				    XmNx, 40, XmNy, 150, XmNheight, 30,
				    XmNwidth, 80, NULL);
  XtAddCallback(find_ok, XmNactivateCallback, find_CB,
		(XtPointer) find_shell);
  find_cancel = XtVaCreateManagedWidget("Cancel", xmPushButtonWidgetClass, 
					find_bb, XmNx, 160, XmNy, 150, 
					XmNheight, 30, XmNwidth, 80, NULL);
  XtAddCallback(find_cancel, XmNactivateCallback, f_cancelCB,
		(XtPointer) find_shell);
  find_help = XtVaCreateManagedWidget("Help", xmPushButtonWidgetClass, find_bb,
				      XmNx, 280, XmNy, 150, XmNheight, 30,
				      XmNwidth, 80, NULL);
  XtAddCallback(find_help, XmNactivateCallback, HelpTopic,
		(XtPointer) FILE_HELP_FIND);
  XtPopup(find_shell, XtGrabNone);
}

void find_CB(Widget w, XtPointer client_data, XtPointer call_data)
{
  Widget menubar_widget, file_pulldown, help_button;
  Widget file_run, file_copy, file_move, file_delete, file_info, file_exit;
  Widget f_shell, f_form;
  char *thispath, *thispattern, *shellcmd, *incoming;
  FILE *fp;
  XmString temp1;
  Arg args[10];

  XtDestroyWidget((Widget) client_data);
  XtVaGetValues(find_tf1, XmNvalue, &thispattern, NULL);
  XtVaGetValues(find_tf2, XmNvalue, &thispath, NULL);
  shellcmd = malloc(sizeof(char)*150);
  incoming = malloc(sizeof(char)*120);

  sprintf(shellcmd, "find %s -name \"%s\"", thispath, thispattern);
#ifdef DEBUG
  printf("%s\n", shellcmd);
#endif

  f_shell = XtVaCreatePopupShell("f_shell", vendorShellWidgetClass, top_level,
				 XmNtitle, "bxfm: Results of Find", NULL);
  f_form = XtVaCreateManagedWidget("f_form", xmFormWidgetClass, f_shell,
				   NULL);
 
  /* Begin menu bar definition. */
  XtSetArg(args[0], XmNtopAttachment, XmATTACH_FORM);
  XtSetArg(args[1], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[2], XmNrightAttachment, XmATTACH_FORM);
  menubar_widget = XmCreateMenuBar(f_form, "menubar", args, 3);
  XtManageChild(menubar_widget);
  file_pulldown = XmCreatePulldownMenu(menubar_widget, "file_pulldown", 
				       NULL, 0);
  file_run = XtVaCreateManagedWidget("file_run", xmPushButtonWidgetClass,
				     file_pulldown, NULL);
  XtAddCallback(file_run, XmNactivateCallback, FindMenuCallback,
		(XtPointer) FILE_MENU_RUN);
  XtVaSetValues(file_run, XmNlabelString, XmStringCreateSimple("Run..."),
		NULL);
  file_copy = XtVaCreateManagedWidget("file_copy", xmPushButtonWidgetClass,
				     file_pulldown, NULL);
  XtAddCallback(file_copy, XmNactivateCallback, FindMenuCallback,
		(XtPointer) FILE_MENU_COPY);
  XtVaSetValues(file_copy, XmNlabelString, XmStringCreateSimple("Copy"),
		NULL);
  file_move = XtVaCreateManagedWidget("file_move", xmPushButtonWidgetClass,
				     file_pulldown, NULL);
  XtAddCallback(file_move, XmNactivateCallback, FindMenuCallback,
		(XtPointer) FILE_MENU_MOVE);
  XtVaSetValues(file_move, XmNlabelString, XmStringCreateSimple("Move"),
		NULL);
  file_delete = XtVaCreateManagedWidget("file_delete", xmPushButtonWidgetClass,
				     file_pulldown, NULL);
  XtAddCallback(file_delete, XmNactivateCallback, FindMenuCallback,
		(XtPointer) FILE_MENU_DELETE);
  XtVaSetValues(file_delete, XmNlabelString, XmStringCreateSimple("Delete"),
		NULL);
  file_info = XtVaCreateManagedWidget("file_info", xmPushButtonWidgetClass,
				     file_pulldown, NULL);
  XtAddCallback(file_info, XmNactivateCallback, FindMenuCallback,
		(XtPointer) FILE_MENU_INFO);
  XtVaSetValues(file_info, XmNlabelString, XmStringCreateSimple("Info"),
		NULL);
  file_exit = XtVaCreateManagedWidget("file_close", xmPushButtonWidgetClass,
				      file_pulldown, NULL);
  XtAddCallback(file_exit, XmNactivateCallback, f_cancelCB,
		(XtPointer) f_shell);
  XtVaSetValues(file_exit, XmNlabelString, XmStringCreateSimple("Close"),
		NULL);
  XtVaCreateManagedWidget("File", xmCascadeButtonWidgetClass, menubar_widget,
			  XmNsubMenuId, file_pulldown, NULL);
  help_button = XtVaCreateManagedWidget("Help", xmCascadeButtonWidgetClass,
					menubar_widget, NULL);
  XtAddCallback(help_button, XmNactivateCallback, HelpTopic,
		(XtPointer) FIND_WINDOW);
  XtVaSetValues(menubar_widget, XmNmenuHelpWidget, help_button, NULL);
  /* End menu bar definition. */

  XtSetArg(args[0], XmNwidth, 360);
  XtSetArg(args[1], XmNheight, 360);
  XtSetArg(args[2], XmNselectionPolicy, XmSINGLE_SELECT);
  XtSetArg(args[3], XmNscrollingPolicy, XmAUTOMATIC);
  XtSetArg(args[4], XmNtopAttachment, XmATTACH_WIDGET);
  XtSetArg(args[5], XmNtopWidget, menubar_widget);
  XtSetArg(args[6], XmNbottomAttachment, XmATTACH_FORM);
  XtSetArg(args[7], XmNleftAttachment, XmATTACH_FORM);
  XtSetArg(args[8], XmNrightAttachment, XmATTACH_FORM);
  f_slist = XmCreateScrolledList(f_form, "f_slist", args, 9);

  if ((fp = popen(shellcmd, "r")) == NULL) {
    NoCanDo(DONT_HAVE_FIND);
    return;
  }
  while (fgets(incoming, 120, fp) != NULL) {
#ifdef DEBUG
    printf("%s\n", incoming);
#endif
    XmListAddItem(f_slist, XmStringCreateSimple(incoming), 0);
  }
  pclose(fp);

  XtManageChild(f_slist);

  XtPopup(f_shell, XtGrabNone);

  free(shellcmd);
  free(incoming);
}

/* This function finishes the data received by FindFileTransfer and punts it
 * into the XmContainer. */
void FinishTransfer(Widget w, XtPointer client_data, 
		    XmSelectionCallbackStruct *call_data) {
  XmString mystring;
  char *my_c_str, full_cmdline[250];
  struct stat buf_src, buf_dest;

  if (call_data->target == XmInternAtom(XtDisplay(w), "COMPOUND_TEXT", False))
    {
      mystring = XmCvtCTToXmString(call_data->value);
      XmStringGetLtoR(mystring, XmSTRING_DEFAULT_CHARSET, &my_c_str);
      sprintf(full_cmdline, "cp -R %s %s", my_c_str, current_wd);
#ifdef DEBUG
      fprintf(stderr, "Request: %s\n", full_cmdline);
#endif
      stat(my_c_str, &buf_src);
      stat(current_wd, &buf_dest);
      XtFree(my_c_str);
      XmStringFree(mystring);
      if (!IsReadable(buf_src))
	NoCanDo(CANT_ACCESS_SRC);
      if (!IsWritable(buf_dest))
	NoCanDo(CANT_ACCESS_DEST);
      system(full_cmdline);
      refresh();
    }
}

/* This function accepts stuff dropped onto the main XmContainer from, say,
 * the Find File window */
void FindFileTransfer(Widget w, XtPointer client_data,
		      XmDestinationCallbackStruct *call_data) {
  Atom ctext = XmInternAtom(XtDisplay(w), "COMPOUND_TEXT", False);

  XmTransferValue(call_data->transfer_id, ctext, FinishTransfer,
		  NULL, 0);
}








